#!/bin/bash
# Generated by runner ({{ runnerVersion }}) at {{ timestamp }}
set -e

{% set instance_data = jsonMap(container.instanceData()) %}
{% set cluster_data = jsonMap(container.clusterData()) %}
{% set esMajorVersion = version.major() %}
{% set esMinorVersion = version.minor() %}
{% set esPatchVersion = version.patch() %}
{% set supportsPurge = esMajorVersion >= 6 or (esMajorVersion == 5 and esMinorVersion >= 5) %}

{% if esMajorVersion == 1 %}
INSTALL='/elasticsearch/bin/plugin -install'
{% elif esMajorVersion == 2 %}
INSTALL='/elasticsearch/bin/plugin install'
{% else %}
INSTALL='/elasticsearch/bin/elasticsearch-plugin install --silent --batch'
{% endif %}

if [[ $(id -u) -eq 0 ]]; then
    echo "ERROR: elasticsearch.sh running as uid 0 (root), exiting."
    exit 1
fi

export ES_PATH_CONF="${ES_PATH_CONF:-/app/config}"
export CONF_DIR=$ES_PATH_CONF # bwc for 1.x and bin/elasticsearch-plugin on 5.x
export LOG_DIR="/app/logs"
export DATA_DIR="/app/data"
export USER_BUNDLES_DIR="/app/found_user_bundles"
export USER_BUNDLES_SCRIPTS_DIR="$USER_BUNDLES_DIR/scripts"
export USER_BUNDLES_DICTS_DIR="$USER_BUNDLES_DIR/dictionaries"

{% if esMajorVersion >= 7 %}
XPACK_ROOT="/app/config"
{% elif esMajorVersion == 6 and esMinorVersion >= 3 %}
XPACK_ROOT="/app/config"
{% elif esMajorVersion >= 5 %}
XPACK_ROOT="/app/config/x-pack"
{% endif %}

cd /app

mkdir -p logs
BOOT_LOG_FILE=logs/boot.log
# save stdout and stderr for later
exec 3>&1 4>&2
exec >  >(ts '%Y-%m-%dT%H:%M:%S%z' |tee -a $BOOT_LOG_FILE)
exec 2> >(ts '%Y-%m-%dT%H:%M:%S%z' |tee -a $BOOT_LOG_FILE >&2)

echo "Booting at $(date)"

# Without this setting, glibc will base the number of malloc arenas on the number of cores, which can hurt small processes running on large servers.
# Heroku has done a decent job benchmarking the effects of this setting, weighing performance of mallocs and actual memory used.
# A setting of 4 seems like a good tradeoff for our needs. https://devcenter.heroku.com/articles/testing-cedar-14-memory-use
export MALLOC_ARENA_MAX=4

# Number of GC threads should depend on the size of the node.
# Tiered compilation does not help Elasticsearch much, and reserves 48M memory for additional code caches.
# Disabling it reduces the code cache, however, which is problematic: https://github.com/elastic/cloud/issues/4603
export ES_JAVA_OPTS="-XX:ParallelGCThreads={{ container.parallelGCThreads() }} -XX:ConcGCThreads={{ container.concGCThreads() }}"

{% if esMajorVersion >= 6 %}
    {% if cluster_data.aws_resources.s3.access_key_id and cluster_data.aws_resources.s3.secret_access_key %}
        {% set snapshot_client = cluster_data.aws_resources.s3.client_name|default("default") %}
        rm -f /app/config/elasticsearch.keystore
        /elasticsearch/bin/elasticsearch-keystore create --silent
        {% set s3_access_key_id = cluster_data.aws_resources.s3.access_key_id | replace("'", "") %}
        echo "{{ s3_access_key_id }}" | /elasticsearch/bin/elasticsearch-keystore add --stdin s3.client.{{ snapshot_client }}.access_key
        {% set s3_secret_access_key = cluster_data.aws_resources.s3.secret_access_key | replace("'", "") %}
        echo "{{ s3_secret_access_key }}" | /elasticsearch/bin/elasticsearch-keystore add --stdin s3.client.{{ snapshot_client }}.secret_key
    {% endif %}

    {% if cluster_data.gcp_resources.gcs %}
        {% set snapshot_client = cluster_data.gcp_resources.gcs.client_name|default("default") %}
        rm -f /app/config/elasticsearch.keystore
        /elasticsearch/bin/elasticsearch-keystore create --silent
        export SERVICE_ACCOUNT_FILE="/app/config/service_account.json"
        echo '{{ serviceAccount }}' > $SERVICE_ACCOUNT_FILE
        /elasticsearch/bin/elasticsearch-keystore add-file gcs.client.{{ snapshot_client }}.credentials_file $SERVICE_ACCOUNT_FILE
        rm $SERVICE_ACCOUNT_FILE
    {% endif %}

    cp /elasticsearch/config/jvm.options /app/config/jvm.options
    echo "" >> /app/config/jvm.options
    echo "# Allow aws access key and secret key to exist in the repository settings" >> /app/config/jvm.options
    echo "-Des.allow_insecure_settings=true" >> /app/config/jvm.options

    {% if (esMajorVersion == 6 and esMinorVersion >= 2) or esMajorVersion >= 7 %}
        # 6.2 does gc logging by default, but to /elasticsearch/logs, we use /app/logs instead
        sed -i -e 's!logs/gc.log!/app/logs/gc.log!g' /app/config/jvm.options
        # additionally, lets only keep 2 gc log files around, rather than 32
        sed -i -E -e 's!(tags:filecount|NumberOfGCLogFiles)=32!\1=2!g' /app/config/jvm.options
        # and drop the size from 64MB to 8MB
        sed -i -E -e 's!(FileSize|filesize)=64m!\1=8m!g' /app/config/jvm.options
    {% endif %}
{% endif %}

{% if esMajorVersion <= 2 %}
    export ES_MAX_MEM={{ container.heapSize() }}M
    export ES_MIN_MEM={{ container.heapSize() }}M
    export ES_HEAP_SIZE={{ container.heapSize() }}M
{% else %}
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -Xms{{ container.heapSize() }}M -Xmx{{ container.heapSize() }}M"

    {% if container.heapSize() < 2048 %}
        # The heap isn't big, so switch to unpooled allocator in netty to save some heap
        export ES_JAVA_OPTS="$ES_JAVA_OPTS -Dio.netty.allocator.type=unpooled"
    {% endif %}
{% endif %}

# Tuning for 5.x per https://github.com/elastic/cloud/issues/6153
# NIO poorly handles freeing direct buffers via the garbage collector.  Fixed in 5.5.1+
{% if esMajorVersion == 5 or esMajorVersion == 2 %}
export ES_JAVA_OPTS="$ES_JAVA_OPTS -XX:+ExplicitGCInvokesConcurrent"
    {% if esMajorVersion == 2 or esMinorVersion <= 4 or (esMinorVersion == 5 and esPatchVersion == 0) %}
export ES_JAVA_OPTS="$ES_JAVA_OPTS -XX:-DisableExplicitGC"
    {% endif %}
{% endif %}

# Use quota aware fs if included in this image.
if [ -f /elasticsearch/lib/quota-aware-fs-*.jar ]; then
    echo "Enabling QuotaAwareFileSystemProvider"
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -Djava.nio.file.spi.DefaultFileSystemProvider=co.elastic.cloud.quotaawarefs.QuotaAwareFileSystemProvider"
fi

# 5.0+ handles OOMs itself https://github.com/elastic/elasticsearch/pull/19272
{% if esMajorVersion <= 2 %}
# ES docker images built before 2016-02-26 may not have libjvmkill installed.
if [ -e /usr/lib/libjvmkill.so ]; then
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -agentpath:/usr/lib/libjvmkill.so"
else
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -XX:OnOutOfMemoryError=/app/on-error-exitcode.sh"
fi
{% endif %}

{% if heapDumpOnOOM|default(false) %}
if [ "{{heapDumpEnabledUntil}}" -gt "$(date +%s)" ]; then
    echo "Enabling heap dump on oom"
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=/app/heap_dumps"
fi
{% endif %}

# Less verbose curator
export ES_JAVA_OPTS="$ES_JAVA_OPTS -Dcurator-log-only-first-connection-issue-as-error-level=true"

{% if esMajorVersion < 5 %}
    export PLUGIN_TARGET_DIR="/data/plugins"
    export PLUGIN_SOURCE_DIR="/elasticsearch/plugins"
{% else %}
    {#
        # In 5.x the ability to set plugin.path was removed
        # https://www.elastic.co/guide/en/elasticsearch/reference/5.0/breaking_50_plugins.html#_custom_plugins_path
    #}
    # ref: https://github.com/elastic/elasticsearch/pull/22452
    export ES_JAVA_OPTS="$ES_JAVA_OPTS -Dio.netty.recycler.maxCapacityPerThread=0"
    export PLUGIN_TARGET_DIR="/elasticsearch/plugins"
    export PLUGIN_SOURCE_DIR="/elasticsearch/plugin-archives"
{% endif %}

{% if esMajorVersion >= 6 %}
    # ES 6 and after make use of the java security manager
    # this will create the gelf policy in order to allow gelf appender to work
    # Also when searching for the jar we remove the cli as it does not require
    # to have a custom security policy.
    es_jar=$(ls /elasticsearch/lib/elasticsearch*.jar | grep -v -E "(cli|core|launchers|secure-sm|x-content)")
    es_cli_jar=$(ls /elasticsearch/lib/elasticsearch-cli*.jar || true) # only 6.1.0 and up
    log4j_jar=$(ls /elasticsearch/lib/log4j-core*.jar)
    gelf_jar=$(ls /elasticsearch/lib/log4j2-gelf*.jar)
    echo "Generating gelf security policy file"
    rm -f /app/config/gelf.policy
    for jar in "$es_jar" "$es_cli_jar" "$log4j_jar" "$gelf_jar"; do
        if [ -n "$jar" ] && [ -e "$jar" ]; then
            # tee so the policy will appear in docker logs, too
            cat << POLICY | tee -a /app/config/gelf.policy
grant codeBase "file://${jar}" {
  permission java.net.SocketPermission "*", "connect,resolve";
};
POLICY
        fi
    done

    export ES_JAVA_OPTS="$ES_JAVA_OPTS -Djava.security.policy=file:///app/config/gelf.policy"
    # if we want to debug the security manager and log access and failures, also need to change the logging level
    # export ES_JAVA_OPTS="$ES_JAVA_OPTS -Djava.security.debug=\"access,failure\""
{% endif %}

{% if (esMajorVersion == 5 and ((esMinorVersion == 1 and esPatchVersion >= 2) or (esMinorVersion >= 2))) or (esMajorVersion >= 6) %}
export ES_JAVA_OPTS="$ES_JAVA_OPTS -Des.cgroups.hierarchy.override=/"
{% endif %}

{% if (esMajorVersion == 6 and esMinorVersion >= 3) or esMajorVersion >= 7 %}
export ES_JAVA_OPTS="$ES_JAVA_OPTS -Des.geoip.load_db_on_heap=true"
{% endif %}

{% set plugin_set_identifier = pluginSetIdentifier | replace('"', "") %}
export PLUGIN_SET_MARKER="/data/found_installed_plugins-{{ plugin_set_identifier }}"
export USER_PLUGINS_MARKER_PATH="/data/user_plugins_installed"
if [ ! -e $PLUGIN_SET_MARKER ]; then
    echo "First time booting with this image, verifying ephemeral directories are empty"

    # Assert the data folder doesn't contain any cruft
    if [ -e /data/found_installed_plugins* ]; then
        echo "ERROR: Ephemeral data path contains marker file from different image, did Docker change behaviour?" && false
    fi

    if [ -e $PLUGIN_TARGET_DIR/* ]; then
        echo "ERROR: Plugin install dir is not empty, but plugin marker file was missing" && false
    fi

    rm -rf /app/plugins #This can line be removed when dropping backwards compatibility with ECE 1.0

    # If it is a symlink, make it a regular directory. Most 5.0 to 5.4 images have a symlink here, no longer needed.
    if [ -h /elasticsearch/plugins ]; then
        rm /elasticsearch/plugins
    fi
    mkdir -p $PLUGIN_TARGET_DIR

    export PURGE_CONFIG_POST_INSTALL='true'
fi

{% if esMajorVersion < 5 %}
    {% for plugin_name in instance_data.elasticsearch.plugins %}
        # if the plugin does not exist copy from bundle. Symlinking does not work since 2.x
        {% set clean_plugin_name = plugin_name | replace("'", "") %}
        if [ ! -e $PLUGIN_TARGET_DIR/{{ clean_plugin_name }} ]; then
            cp -r $PLUGIN_SOURCE_DIR/{{ clean_plugin_name }} $PLUGIN_TARGET_DIR/{{ clean_plugin_name }} || true
        fi
    {%  endfor %}
{% else %}
    # Deleting any marker file here as a workaround for supporting 5.x images
    # without ephemeral dir, otherwise listing plugins fail below.
    rm -f /elasticsearch/plugins/.found_installed_plugins*

    mkdir -p $USER_PLUGINS_MARKER_PATH

    # When listing the installed plugins, we remove any plugins that start with a tab character as these plugins are
    # part of a plugin bundle. When removing plugins, we would remove the bundle as Elasticsearch will then remove the
    # associated bundle plugins
    export ALREADY_INSTALLED=$(/elasticsearch/bin/elasticsearch-plugin list | sed '/^\t/d' | sort)

    export TO_BE_REMOVED=$(comm -23 <(echo "$ALREADY_INSTALLED") <(
        for PLUGIN in {{ allPluginsInstalled|replace("\n", "  ") | replace(";", "") }} ; do \
            MARKERFILE=$(find "$USER_PLUGINS_MARKER_PATH" -regex "$USER_PLUGINS_MARKER_PATH/$PLUGIN-[a-f0-9]+")
            if [ -n "$MARKERFILE" ] ; then
                cat "$MARKERFILE"
            else
                echo $PLUGIN
            fi
        done | sort
    ))
    {% set plugins_to_add_clean = instance_data.elasticsearch.plugins|join('\n') | replace("'", "") %}
    export TO_BE_ADDED=$(comm -13 <(echo "$ALREADY_INSTALLED") <(echo '{{ plugins_to_add_clean }}' | sort))

    for PLUGIN in $TO_BE_REMOVED; do \
        echo "Removing unselected plugin: $PLUGIN"
        {% if supportsPurge %}
            /elasticsearch/bin/elasticsearch-plugin remove --purge $PLUGIN
        {% else %}
            /elasticsearch/bin/elasticsearch-plugin remove $PLUGIN
            if [ -e /app/config/$PLUGIN ]; then
                echo "Purging config at: /app/config/$PLUGIN"
                rm -r "/app/config/$PLUGIN"
            fi
        {% endif %}
        # Wipe out any marker file that may have referenced this plugin
        grep -Rl "^$PLUGIN$" $USER_PLUGINS_MARKER_PATH | xargs rm -f
    done

    for PLUGIN in $TO_BE_ADDED; do \
        echo "Adding selected plugin: $PLUGIN"
        $INSTALL file://$(readlink -f $PLUGIN_SOURCE_DIR/$PLUGIN-*.zip)
    done
{% endif %}

echo "Installing user plugins."
{% for user_plugin in instance_data.elasticsearch.user_plugins %}
    {% set name = user_plugin.name | replace("'", "") %}
    {% set url = user_plugin.url | replace("'", "") %}
    {% if url != user_plugin.url %}
      echo "Single quotes were replaced from the plugin's URL, this might result in an error when starting the container"
    {% endif %}
    # Installing user plugin: {{ name }}
    {% if esMajorVersion < 5 %} {# Version allows rename user plugins to configured name #}
        if [ ! -e $PLUGIN_TARGET_DIR/{{ name }} ] && [ ! -e $PLUGIN_TARGET_DIR/{{ name }}/.found_installed_plugin ]; then
            echo "Installing {{ name }}..."
            rm -rf $PLUGIN_TARGET_DIR/{{ name }} || true

            {% if esMajorVersion == 1 %}
                JAVA_OPTS=-Des.path.plugins=$PLUGIN_TARGET_DIR $INSTALL {{ name }} -url '{{ url }}' && touch $PLUGIN_TARGET_DIR/{{ name }}/.found_installed_plugin || true
            {% else %}
                wget --no-verbose -O $PLUGIN_TARGET_DIR/{{ name }}.zip '{{ url }}' \
                    && unzip -o $PLUGIN_TARGET_DIR/{{ name }}.zip -d $PLUGIN_TARGET_DIR/{{ name }} \
                    || false
                rm $PLUGIN_TARGET_DIR/{{ name }}.zip
            {% endif %}

            echo "Installed {{ name }}."
        else
            if [ -e $PLUGIN_TARGET_DIR/{{ name }}/.found_installed_plugin ]; then
                echo "Already installed {{ name }}."
            else
                echo "Not installing {{ name }} because it already exists as a built-in, enabled plugin"
            fi
        fi
    {% else %} {# Record real name in marker file #}
        MARKER_FILE="$USER_PLUGINS_MARKER_PATH/{{ name }}-$(echo '{{ url }}' | md5sum | awk '{ print $1 }' )"
        if [ ! -e "$MARKER_FILE" ]; then
            export OTHER_MARKER=$(find $USER_PLUGINS_MARKER_PATH -regex $USER_PLUGINS_MARKER_PATH/{{ name }}-[a-f0-9]+)
            if [ -n "$OTHER_MARKER" ]; then
                echo "Uninstalling previous version of user plugin: {{ name }}"
                /elasticsearch/bin/elasticsearch-plugin remove $(cat "$OTHER_MARKER")
                rm "$OTHER_MARKER"
            fi
            echo "Installing {{ name }}..."
            wget --no-verbose -O $USER_PLUGINS_MARKER_PATH/{{ name }}.zip '{{ url }}'
            {% if esMajorVersion < 6 or (esMajorVersion == 6 and esMinorVersion < 3) %} {# Before 6.3, plugins must have an "elasticsearch" dir at the root #}
            declare PLUGIN_DESCRIPTOR_PATH=elasticsearch/plugin-descriptor.properties
            {% else %} {# Since 6.3, there is no "elasticsearch" root dir anymore #}
            declare PLUGIN_DESCRIPTOR_PATH=plugin-descriptor.properties
            {% endif %}
            declare REAL_NAME=$(
                unzip -p $USER_PLUGINS_MARKER_PATH/{{ name }}.zip $PLUGIN_DESCRIPTOR_PATH \
                | grep '^ *name *[=:]' | tr : = | cut -d'=' -f2- | awk '{$1=$1};1'
            )
            $INSTALL file://$(readlink -f $USER_PLUGINS_MARKER_PATH/{{ name }}.zip) && echo "$REAL_NAME" > "$MARKER_FILE"
            rm $USER_PLUGINS_MARKER_PATH/{{ name }}.zip
            echo "Installed {{ name }}."
        else
            echo "Already installed {{ name }}."
        fi
    {% endif %}
{% endfor %}

{% if esMajorVersion < 5 %} {# For the older versions, user plugins are renamed to configured name during install #}
declare CURRENTLY_INSTALLED="{{ allPluginsInstalled }}"
{% else %} {# Custom plugins are not renamed as part of install since 5.0 so we need to record the actual name and not what was configured in plan. #}

# When listing the installed plugins, we remove any plugins that start with a tab character as these plugins are
# part of a plugin bundle. When purging plugins, we would remove the bundle as Elasticsearch will then remove the
# associated bundle plugins
declare CURRENTLY_INSTALLED=$(/elasticsearch/bin/elasticsearch-plugin list | sed '/^\t/d' | sort)
{% endif %}

# This block ensures that if a plugin was removed at the same time as the docker image was upgraded, we
# don't leave config lingering. Mostly to ensure it doesn't create trouble for a future reinstall of that plugin.
# Unlike other cleanups triggered by upgrade, this is done post plugin install in order to compare against real plugin names
if [ "$PURGE_CONFIG_POST_INSTALL" == 'true' ] && [ -e /app/plugins_installed ]; then
    export NEEDS_CONFIG_PURGE=$(comm -23 /app/plugins_installed <(echo "$CURRENTLY_INSTALLED"))
    for PLUGIN in $NEEDS_CONFIG_PURGE; do \
        if [ -e /app/config/$PLUGIN ]; then
            {% if supportsPurge %}
                echo "Detected lingering config for previously installed plugin: $PLUGIN, purging config."
                /elasticsearch/bin/elasticsearch-plugin remove --purge $PLUGIN
            {% else %}
                echo "Detected lingering config for previously installed plugin, removing: /app/config/$PLUGIN"
                rm -r "/app/config/$PLUGIN"
            {% endif %}
        fi
    done
fi

#Record plugins installed, to enable purging config of any plugin implicitly removed during an in-place upgrade
echo "$CURRENTLY_INSTALLED" > /app/plugins_installed

# Indicate that plugins should be ok for this version, so we don't wipe everything above
touch $PLUGIN_SET_MARKER

echo "Installing user bundles."

mkdir -p "$USER_BUNDLES_DIR/.found-cached-downloads"
{% if not container.shouldInstallUserBundles() %}
echo "Skipping installation of user bundles due to allocation configuration."
{% elif not instance_data.elasticsearch.user_bundles %}
echo "No user bundles defined"
{% else %}
{% for user_bundle in instance_data.elasticsearch.user_bundles %}
    {% set name = user_bundle.name | replace("'", "") %}
    if [ ! -e $USER_BUNDLES_DIR/.found_installed_bundle_{{ name }} ]; then
        echo "Installing bundle [{{ name }}]..."
        {% set clean_bundle_url = user_bundle['url'] | replace("'", "") %}
        {% if clean_bundle_url != user_bundle['url'] %}
          echo "A single quote was filtered from the URL of the bundle. This might result on a container not starting, but could be a signal of an attack"
        {% endif %}
        {% set clean_bundle_url_hash = clean_bundle_url | md5 %}

        if [ ! -e $USER_BUNDLES_DIR/.found-cached-downloads/{{ name }}-{{ clean_bundle_url_hash }}.zip ]; then
          echo "Downloading bundle [{{ name }}]"
          if ! wget --no-use-server-timestamps -O $USER_BUNDLES_DIR/.found-cached-downloads/{{ name }}-{{ clean_bundle_url_hash }}.zip '{{ clean_bundle_url }}'; then
            echo "Could not download bundle [{{ name }}]"
            rm -f $USER_BUNDLES_DIR/.found-cached-downloads/{{ name }}-{{ clean_bundle_url_hash }}.zip
            exit 1
          fi

          echo "Validating bundle [{{ name }}]"
          if ! unzip -t $USER_BUNDLES_DIR/.found-cached-downloads/{{ name }}-{{ clean_bundle_url_hash }}.zip; then
            echo "Bundle [{{ name }}] is not a zip file"
            rm -f $USER_BUNDLES_DIR/.found-cached-downloads/{{ name }}-{{ clean_bundle_url_hash }}.zip
            exit 1
          fi

          echo "Clearing expired cached versions of bundle [{{ name }}]"
          ls -t $USER_BUNDLES_DIR/.found-cached-downloads/{{ name }}-*.zip | tail -n +3 | xargs rm -f
        fi

        echo "Copying data from bundle [{{ name }}]"
        unzip -o $USER_BUNDLES_DIR/.found-cached-downloads/{{ name }}-{{ clean_bundle_url_hash }}.zip -d $USER_BUNDLES_DIR/

        echo "Installed bundle [{{ name }}]."
        echo "{{ clean_bundle_url_hash }}" > $USER_BUNDLES_DIR/.found_installed_bundle_{{ name }}
    else
        echo "Already installed bundle [{{ name }}]."
    fi
{% endfor %}

echo "Ensuring read/write permissions on all bundles"
chmod u+rw -R $USER_BUNDLES_DIR/*

{% if installBundlesFromStartScript|default(true) %}
# Copy the scripts directory to the found_user_bundles plugin
[[ -h $ES_PATH_CONF/scripts ]] && rm $ES_PATH_CONF/scripts
if [ -e $USER_BUNDLES_SCRIPTS_DIR ]; then
    cp -r $USER_BUNDLES_SCRIPTS_DIR $ES_PATH_CONF/scripts
fi

# Copy the dictionaries directory from the found_user_bundles
[[ -h $ES_PATH_CONF/dictionaries ]] && rm $ES_PATH_CONF/dictionaries
if [ -e $USER_BUNDLES_DICTS_DIR ]; then
    # We copy the dictionaries directly to /app/config. We need `-n` in order not to override existing files
    echo "Installing dictionaries"
    cp -rn $USER_BUNDLES_DICTS_DIR/* $ES_PATH_CONF
fi

# Maybe an old-style bundle
old_style_items=$(find $USER_BUNDLES_DIR -maxdepth 1 \
        -not -path $USER_BUNDLES_DIR \
        -not -path $USER_BUNDLES_SCRIPTS_DIR \
        -not -path $USER_BUNDLES_DICTS_DIR \
        -not -path "$USER_BUNDLES_DIR/\.found*")
for item in $old_style_items; do
    echo "Installing old-style-bundle dictionary: $item"
    cp -rn "$item" $ES_PATH_CONF
done
{% endif %}
{% endif %}

{% if esMajorVersion >= 5 %}
export ES_JAVA_OPTS="$ES_JAVA_OPTS -Dlog4j2.formatMsgNoLookups=true"
{% endif %}

echo "Done installing plugins and bundles, verifying required config files."

{% if esMajorVersion >= 5 %}
if [ ! -f $XPACK_ROOT/system_key ]; then
    echo "Security System Key file not found ($XPACK_ROOT/system_key)"
    exit 1
fi
{% endif %}

echo "Done verifying required config files, starting Elasticsearch. See Elasticsearch logs for further output."

{% if esMajorVersion == 1 %}
export CONFIG_OPTIONS="-Des.config=/app/config/elasticsearch.yml"
{% elif esMajorVersion == 2 %}
export CONFIG_OPTIONS="-Des.default.path.logs=${LOG_DIR} -Des.default.path.data=${DATA_DIR} -Des.default.path.conf=${ES_PATH_CONF}"
{% elif esMajorVersion == 5 %}
export CONFIG_OPTIONS="-Epath.logs=${LOG_DIR} -Epath.data=${DATA_DIR} -Epath.conf=${ES_PATH_CONF}"
{% else %}
# ES 6
export CONFIG_OPTIONS="-Epath.logs=${LOG_DIR} -Epath.data=${DATA_DIR}"
{% endif %}

# used by metricbeat
export ECE_COMPONENT="elasticsearch"
export ECE_CLUSTER_NAME="{{ container.id().group() }}"
export ECE_INSTANCE_ID="{{ container.id().name() }}"
export ECE_CLUSTER_VERSION="{{ container.versionString() }}"

# restore stdout and stderr
exec 1>&3 2>&4

# set cluster data environment variables, this is done to allow being able to
# modify the elasticsearch environment (for example java configuration and the like)
# without having to vacate the node, as Docker environment properties can't be modified
# while these properties are set by the allocator
{% for environment_variable in cluster_data.overrides.elasticsearchEnv %}
export {{environment_variable}}
{% endfor %}

# Do not abort script if elasticsearch returns error code
set +e
{% if esMajorVersion <= 2 %}
/elasticsearch/bin/elasticsearch -Dfound.cluster_name={{ container.id().group() }} -Dfound.instance_name={{ container.id().name() }} -p /app/es.pid $CONFIG_OPTIONS $*
{% else %}
/elasticsearch/bin/elasticsearch -p /app/es.pid $CONFIG_OPTIONS $*
{% endif %}
ES_EXIT_CODE=$?
if [ "$ES_EXIT_CODE" -ge "1" ]; then
    echo "Elasticsearch exited with status $ES_EXIT_CODE, running ./on-error-exitcode.sh $ES_EXIT_CODE"
    ./on-error-exitcode.sh $ES_EXIT_CODE
fi
exit $ES_EXIT_CODE
