#!/bin/bash -ex
# write stdout/stderr to the oom.log file
exec &>>/app/logs/error-exitcode.log

WIPE_INSTANCE_STATE_PATH=/usr/local/bin/wipe-instance-state

CODE=$1
WHEN=$(date)
echo "Error exitcode ${CODE} at ${WHEN}"

if [ $# -eq 0 ]; then
    # If no arguments provided, it was invoked by the JVM due to the
    ##  -XX:OnOutOfMemoryError flag. This will actually trigger this
    ##  script to be invoked a second time due to the JVM _actually_
    ##  dying and elasticsearch.sh completing it's execution.
    ##  The second invocation will have the exit code 137.
    killall -9 java
    exit 0
fi

wipe_instance_state() {
    local extra_args=${1}
    ${WIPE_INSTANCE_STATE_PATH} --config /app/config/elasticsearch.yml --script-timeout 12000 ${extra_args}
    echo "slain at ${WHEN} with ${CODE}" >> /app/logs/slain.log
}

if [ -e ${WIPE_INSTANCE_STATE_PATH} ]; then
    case ${CODE} in
          1)  # ElasticsearchUncaughtExceptionHandler: Unknown fatal error
            wipe_instance_state
            ;;
          3)  # FileAlreadyExistsException - GH issue #3831
            wipe_instance_state
            ;;
         78)  # Bootstrap check failure - GH issue #3831
            wipe_instance_state
            ;;
        124)  # ElasticsearchUncaughtExceptionHandler: Serious I/O error
            wipe_instance_state
            ;;
        125)  # ElasticsearchUncaughtExceptionHandler: Unknown virtual machine error
            wipe_instance_state
            ;;
        126)  # ElasticsearchUncaughtExceptionHandler: Stack overflow error
            wipe_instance_state
            ;;
        127)  # ElasticsearchUncaughtExceptionHandler: Out of memory error
            wipe_instance_state --slain
            ;;
        128)  # ElasticsearchUncaughtExceptionHandler: VM internal error
            wipe_instance_state
            ;;
        158)  # 128 + 30 - killed by jvmkiller agent
            wipe_instance_state --slain
            ;;
        137)  # 128 + 9 - killed by kernel oom-killer
            wipe_instance_state --slain
            ;;
        134)  # 128 + 6 - segfault
            wipe_instance_state
            ;;
        *)
            echo "No idea what to do with error code $CODE"
            ;;
    esac
else
    echo "Not wiping instance state because the command wipe-instance-state was not found"
fi

HEAP_DUMPS=$(find /app/heap_dumps/ -maxdepth 1 -type f -regex '.*\.hprof')
if [ -n "$HEAP_DUMPS" ]; then
    echo -n "Compressing heap dumps "
    if [[ $(type -P brotli) ]]; then
        # Brotli is preferred when available because it has a better speed to compression ratio.
        echo "with brotli"
        echo "$HEAP_DUMPS" | xargs brotli --rm
        mv /app/heap_dumps/*.hprof.br /app/heap_dumps/compressed/
    elif [[ $(type -P pigz) ]]; then
        # pigz creates regular gz archives, but supports utilizing multiple cores
        echo "with pigz"
        echo "$HEAP_DUMPS" | xargs pigz -p `expr $(nproc) / 2`
        mv /app/heap_dumps/*.hprof.gz /app/heap_dumps/compressed/
    else
        echo "with gzip"
        echo "$HEAP_DUMPS" | xargs gzip
        mv /app/heap_dumps/*.hprof.gz /app/heap_dumps/compressed/
    fi
fi


echo "exiting on-error-exitcode.sh at $(date)"
exit 0
